require 'file-tail'

class LogMagic
  def initialize(log_file_name)
    @log_file_name = log_file_name
    reset_refiners
  end

  def initialize_listener
    @log_listener = LogListener.new(@log_file_name)
  end

  def start
    @log_listener.listen do |line|
      add_line_to_refiners(line)
      check_for_refiner_match do |refiner|
        puts refiner.compute
      end
      reset_refiners
    end
  end

  def add_line_to_refiners(line)
    @refiners.each do |refiner|
      refiner.lines << line
    end
  end

  def check_for_refiner_match
    @refiners.any? do |refiner|
      if refiner.matches?
        yield refiner
      end
    end
  end

  def add_searchkick_refiner
    @refiners << SearchkickRefiner.new
  end

  def add_mysql_refiner
    @refiners << MySqlRefiner.new
  end

  def reset_refiners
    @refiners = []
    add_searchkick_refiner
    add_mysql_refiner
  end
end

require 'active_support/all'

require 'log_magic/utils/json_utils'
require 'log_magic/utils/templating_utils'
require 'log_magic/utils/port_utils'

require 'log_magic/log_listener'

require 'log_magic/refiners/base_refiner'
require 'log_magic/refiners/searchkick_refiner'
require 'log_magic/refiners/mysql_refiner'

require 'log_magic/printers/searchkick_printer'
require 'log_magic/printers/mysql_printer'

require 'log_magic/renderers/hover_renderer'

require 'log_magic/explainers/base_explainer'
require 'log_magic/explainers/searchkick_explainer'
require 'log_magic/explainers/mysql_explainer'


['searchkick', 'mysql'].each do |explainer_type|
  explainer_files = \
  File.expand_path(File.join(__FILE__, '..', "/log_magic/explainers/#{explainer_type}/"))

  Dir.foreach(explainer_files).each do |explainer|
    next if explainer == '.' || explainer == '..'
    require File.join('log_magic', 'explainers', explainer_type, explainer)
  end
end

require 'log_magic/persistance_layer'
